@@ -434,7 +434,7 @@ class Scratch2Py():
             except:
                 return 'Sorry, there was an error.'
 
-    class cloudDatabase:
+    class scratchDatabase:
         def __init__(self, pid):
             self.id = pid
             self.username = uname
@@ -587,6 +587,157 @@ class Scratch2Py():
                         tosend = self.__encode('Invalid input.')
                         self.__setCloudVar('Return', tosend)                 
 
+    class turbowarpDatabase:
+        def __init__(self, pid):
+            self.id = pid
+            self.username = uname
+            ws.connect('wss://clouddata.turbowarp.org',
+                    origin='https://turbowarp.org', enable_multithread=True)
+            ws.send(json.dumps({
+                'method': 'handshake',
+                'user': self.username,
+                'project_id': str(self.id)
+            }) + '\n')
+
+        def __decode(self, text):
+            decoded = ""
+            y = 0
+            for i in range(0,len(text)//2):
+                x = chars[int(str(text[y])+str(text[int(y)+1]))-9]
+                decoded = str(decoded)+str(x)
+                y += 2
+            return decoded
+
+        def __encode(self, text):
+            global chars
+            text = text.lower()
+            encoded = ""
+            length = int(len(text))
+            for i in range(0,length):
+                try:
+                    x = int(chars.index(text[i])+int(9))
+                    encoded = encoded + str(x)
+                except ValueError:
+                    logging.error('Character not supported')
+            return encoded
+    
+        
+        def __readCloudVar(self, variable):
+            ws.send(json.dumps({
+                'method': 'get',
+                'project_id': str(turbowarpid)
+                }) + '\n')
+            data = ws.recv()
+            data = data.split('\n')
+            result = []
+            for i in data:
+                result.append(json.loads(i))
+            for i in result:
+                if i['name'] == '☁ ' + variable:
+                    return i['value']
+            return 'Variable not found.'
+        
+        def __setCloudVar(self, variable, value):
+            ws.send(json.dumps({
+                'method': 'set',
+                'name': '☁ ' + variable,
+                'value': str(value),
+                'user': self.username,
+                'project_id': str(turbowarpid)
+                }) + '\n')
+
+
+        def __readCloudVar(self, name, limit="1000"):
+                try:
+                    resp = requests.get("https://clouddata.scratch.mit.edu/logs?projectid=" +
+                                str(self.id)+"&limit="+str(limit)+"&offset=0").json()
+                    for i in resp:
+                        x = i['name']
+                        if x == ('☁ ' + str(name)):
+                            return i['value']
+                except json.decoder.JSONDecodeError:
+                    resp = requests.get("https://clouddata.scratch.mit.edu/logs?projectid=" +
+                                str(self.id)+"&limit="+str(limit)+"&offset=0").json()
+                    for i in resp:
+                        x = i['name']
+                        if x == ('☁ ' + str(name)):
+                            return i['value']
+
+        def startLoop(self):
+            data = []
+            while True:
+                encodedMethod = self.__readCloudVar('Method')
+                if encodedMethod != None:
+                    Method = self.decode(encodedMethod)
+                if Method == "set":
+                    encodedSend = self.__readCloudVar('Send')
+                    Send = str(self.__decode(encodedSend))
+                    encodedVal = self.__readCloudVar('Data')
+                    Val = str(self.__decode(encodedVal))
+                    intVal = self.__decode(encodedVal)
+                    c = 0
+                    for i in Send:
+                        if str(i) in "1234567890":
+                            c = int(c)+1
+                    if c == len(Send):
+                        if int(Send) > len(data):
+                            if int(Send) == int(len(data))+1:
+                                data.append(intVal)
+                                logging.info('Data added.')
+                                tosend = self.__encode('Data added.')
+                                self.__setCloudVar('Return',tosend)
+                                self.__setCloudVar('Method','')
+                            else:
+                                while len(data) != int(Send)-1:
+                                    data.append('none')
+                                data.append(intVal)
+                                logging.info('Data added.')
+                                tosend = self.__encode('Data added.')
+                                self.__setCloudVar('Return',tosend)
+                                self.__setCloudVar('Method','')
+                        else:
+                            data.pop(int(Send)-1)
+                            data.insert(int(Send), intVal)
+                            logging.info('Data added.')
+                            tosend = self.__encode('Data added.')
+                            self.__setCloudVar('Return',tosend)
+                            self.__setCloudVar('Method','')
+                    else:
+                        tosend = self.__encode('Invalid input. Variable name must be int.')
+                        self.__setCloudVar('Return', tosend)
+                if Method == "get":
+                    encodedSend = self.__readCloudVar('Send')
+                    Send = self.__decode(encodedSend)
+                    c = 0
+                    for i in Send:
+                        if str(i) in "1234567890":
+                            c = int(c)+1
+                    if c == len(Send) and int(Send) > 0 and int(Send) < int(len(data))+1:
+                        tosend = self.__encode(data[int(Send)-1])
+                        self.__setCloudVar('Return',tosend)
+                        logging.info('Data sent.')
+                        self.__setCloudVar('Method','')
+                    else:
+                        tosend = self.__encode('Invalid input.')
+                        self.__setCloudVar('Return', tosend)
+                if Method == "delete":
+                    encodedSend = self.__readCloudVar('Send')
+                    Send = self.__decode(encodedSend)
+                    c = 0
+                    for i in Send:
+                        if str(i) in "1234567890":
+                            c = int(c)+1
+                    if c == len(Send) and int(Send) > 0 and int(Send) < int(len(data))+1:
+                        data.pop(int(Send)-1)
+                        data.insert(int(Send)-1,'none')
+                        logging.info('Variable deleted.')
+                        tosend = self.__encode('Variable deleted.')
+                        self.__setCloudVar('Return',tosend)
+                    else:
+                        tosend = self.__encode('Invalid input.')
+                        self.__setCloudVar('Return', tosend)                 
+
+    
     class turbowarpConnect:
         def __init__(self, pid):    
             global ws
